# coding=utf-8
'''provide class Path of type `str`.'''

import os


class Path(str):
    '''
    convenient to join file path in a chain manner:
    s = s.join('a', 'b').join('..')
    '''

    def join(self, *args, **kw):
        '''
        join file path, call this method like os.path.join

        Example: Path('./dir/sub').join('..')  ->  './dir'
        '''
        return Path(os.path.normpath(os.path.join(self, *args, **kw)))


__all__ = ['Path']



# coding=utf-8

'''
read/write .properties file in a simple line-oriented format.

Note: the code takes Java class `java.util.Properties` for reference.

properties data are key-value pairs stored in a dictionary.

common usage in file I/O:

properties = load(file)
store(file, properties)

main features:
* seperate key and value by one of `=`, `:`, ` `, `\\t`
* ignore whitespaces leading in a line or around `=` or `:`
* comment line begin with `#` or `!`
* escape unicode by `\\uxxxx`
* escape special characters by adding `\\`

others:
* data line ends with '\\' discard the line break

differences with Java:
* store method will not write datetime comment
'''

from collections import Iterable
import os
import re
import sys

# 兼容 python3
if sys.version_info.major == 3:
    def hasNext(o):
        return hasattr(o, '__next__')
    xrange = range
else:
    def hasNext(o):
        return hasattr(o, 'next')

    def next(o):
        return o.next()


def load(iterable, properties=None):
    '''
    read properties in line-oriented format and store in the dictionary.

    Note: key and value will be of unicode type if the input data
    has \\uxxxx that will be decoded as unicode char.

    @param  iterable    a iterable object to get `str` data
                        (normally the file object)
    @param  properties  dictionary to store data,
                        defaults None and the data with return
    @return             dictionary stores the key-value data

    @call  class LineReader
    @call  #loadSingle(string)

    ---detail from Java document---
     * Properties are processed in terms of lines. There are two
     * kinds of line, natural lines and logical lines.
     * A natural line is defined as a line of
     * characters that is terminated either by a set of line terminator
     * characters ('\\n' or '\\r' or '\\r\\n')
     * or by the end of the stream. A natural line may be either a blank line,
     * a comment line, or hold all or some of a key-element pair. A logical
     * line holds all the data of a key-element pair, which may be spread
     * out across several adjacent natural lines by escaping
     * the line terminator sequence with a backslash character
     * '\\'.  Note that a comment line cannot be extended
     * in this manner; every natural line that is a comment must have
     * its own comment indicator, as described below. Lines are read from
     * input until the end of the *iteration* is reached.
     * 
     * A natural line that contains only white space characters is
     * considered blank and is ignored.  A comment line has an ASCII
     * '#' or '!' as its first non-white
     * space character; comment lines are also ignored and do not
     * encode key-element information.  In addition to line
     * terminators, this format considers the characters space
     * (' '), tab
     * ('\\t'), and form feed
     * ('\\f') to be white
     * space.
     * 
     * If a logical line is spread across several natural lines, the
     * backslash escaping the line terminator sequence, the line
     * terminator sequence, and any white space at the start of the
     * following line have no affect on the key or element values.
     * The remainder of the discussion of key and element parsing
     * (when loading) will assume all the characters constituting
     * the key and element appear on a single natural line after
     * line continuation characters have been removed. Note that
     * it is not sufficient to only examine the character
     * preceding a line terminator sequence to decide if the line
     * terminator is escaped; there must be an odd number of
     * contiguous backslashes for the line terminator to be escaped.
     * Since the input is processed from left to right, a
     * non-zero even number of 2n contiguous backslashes
     * before a line terminator (or elsewhere) encodes n
     * backslashes after escape processing.
     * 
     * The key contains all of the characters in the line starting
     * with the first non-white space character and up to, but not
     * including, the first unescaped '=',
     * ':', or white space character other than a line
     * terminator. All of these key termination characters may be
     * included in the key by escaping them with a preceding backslash
     * character; for example,
     * 
     * \\:\\=
     * 
     * would be the two-character key ":=".  Line
     * terminator characters can be included using \\r and
     * \\n escape sequences. Any white space after the
     * key is skipped; if the first non-white space character after
     * the key is '=' or ':', then it is
     * ignored and any white space characters after it are also
     * skipped.  All remaining characters on the line become part of
     * the associated element string; if there are no remaining
     * characters, the element is the empty string
     * "".  Once the raw character sequences
     * constituting the key and element are identified, escape
     * processing is performed as described above.
     * 
     * As an example, each of the following three lines specifies the key
     * "Truth" and the associated element value
     * "Beauty":
     * 
     * Truth = Beauty
     *  Truth:Beauty
     * Truth                    :Beauty
     * 
     * As another example, the following three lines specify a single
     * property:
     * 
     * fruits                           apple, banana, pear, \
     *                                  cantaloupe, watermelon, \
     *                                  kiwi, mango
     *
     * The key is "fruits" and the associated element is:
     * "apple, banana, pear, cantaloupe, watermelon, kiwi, mango"
     * Note that a space appears before each '\\' so that a space
     * will appear after each comma in the final result; the '\\',
     * line terminator, and leading white space on the continuation line are
     * merely discarded and are not replaced by one or more other
     * characters.
     * 
     * As a third example, the line:
     * cheeses
     * 
     * specifies that the key is "cheeses" and the associated
     * element is the empty string "".
     * 
     * Characters in keys and elements can be represented in escape
     * sequences similar to those used for character and string literals
     * (see sections 3.3 and 3.10.6 of
     * The Java™ Language Specification).
     *
     * The differences from the character escape sequences and Unicode
     * escapes used for characters and strings are:
     *
     *  *   Octal escapes are not recognized.
     *
     *  *   The character sequence '\\b' does not
     *      represent a backspace character.
     *
     *  *   The method does not treat a backslash character,
     *      '\\', before a non-valid escape character as an
     *      error; the backslash is silently dropped.  For example, in a
     *      Java string the sequence "\\z" would cause a
     *      compile time error.  In contrast, this method silently drops
     *      the backslash.  Therefore, this method treats the two character
     *      sequence "\\b" as equivalent to the single
     *      character 'b'.
     *
     *  *   Escapes are not necessary for single and double quotes;
     *      however, by the rule above, single and double quote characters
     *      preceded by a backslash still yield single and double quote
     *      characters, respectively.
     *
     *  *   Only a single 'u' character is allowed in a Unicode escape
     *      sequence.
    '''
    if properties == None:
        properties = {}
    for line in LineReader(iterable):
        key, value = loadSingle(line)
        properties[key] = value
    return properties


def loadSingle(string):
    '''
    read a key-value pair from data of one key-value.
    this identifies the key and value, seperates them and does escape processing.

    Note: to provide independent use, this first skips leading whitespaces
    which will normally be done by LineReader.

    @example
    loadSingle('key = value') -> 'key', 'value'

    @param  string  one data of key-value pair
    @return key, value

    @call  #identifyKeyValue(line)
    @call  #loadConvert(inp, off, length)

    @see  load(iterable, properties=None)
    '''
    for i in xrange(len(string)):
        if string[i] == ' ' or string[i] == '\t' or string[i] == '\f':
            continue
        break
    if i > 0:
        string = string[i:]
    keyEnd, valueStart = identifyKeyValue(string)
    key = loadConvert(string, 0, keyEnd)
    value = loadConvert(string, valueStart, len(string) - valueStart)
    return key, value


_loadSingle2_pattern = re.compile(
    r'^[ \t\f]*((?:\\\\|\\=|\\:|[^ \t\f])+?)(?:[ \t\f]*(?::|=)[ \t\f]*|[ \t\f]+)([\s\S]*)$')


def loadSingle2(string):
    '''
    regex implement
    '''
    for i in xrange(len(string)):
        if string[i] == ' ' or string[i] == '\t' or string[i] == '\f':
            continue
        break
    if i > 0:
        string = string[i:]
    m = re.search(_loadSingle2_pattern, string)
    key = loadConvert(m.group(1))
    value = loadConvert(m.group(2))
    return key, value


def identifyKeyValue(string):
    '''
    identify key-end and value-start position
    from data start immediately with the key.

    the data should not have leading whitespace
    since whitespace will be read as seperator,
    then the key will be empty.

    @example
    identifyKeyValue('key=value') -> 3, 4
    identifyKeyValue('key = value') -> 3, 6
    '''
    limit = len(string)
    keyEnd = 0
    valueStart = limit
    hasSep = False
    precedingBackslash = False
    while (keyEnd < limit):
        c = string[keyEnd]
        # need check if escaped.
        if (c == '=' or c == ':') and not precedingBackslash:
            valueStart = keyEnd + 1
            hasSep = True
            break
        elif ((c == ' ' or c == '\t' or c == '\f') and
              not precedingBackslash):
            valueStart = keyEnd + 1
            break
        if c == '\\':
            precedingBackslash = not precedingBackslash
        else:
            precedingBackslash = False
        keyEnd += 1
    while (valueStart < limit):
        c = string[valueStart]
        if c != ' ' and c != '\t' and c != '\f':
            if not hasSep and (c == '=' or c == ':'):
                hasSep = True
            else:
                break
        valueStart += 1
    return keyEnd, valueStart


def loadConvert(string, off=0, length=None):
    '''
    Converts encoded \\uxxxx to unicode chars
    and changes special saved chars to their original forms

    @example
    loadConvert('\\==', 0, 2) -> '='

    @param  string  string to process
    @param  off     offset, the beginning index to process, defaults 0
    @param  length  length of the string begin from offset to process,
                    defaults to process to the end
    @return         processed string
    '''
    out = []
    if length == None:
        end = len(string)
    else:
        end = off + length
    while (off < end):
        aChar = string[off]
        off += 1
        if aChar == '\\':
            aChar = string[off]
            off += 1
            if aChar == 'u':
                aChar = (u'\\u' + string[off:off+4]
                         ).encode().decode('unicode-escape')
                off += 4
            else:
                if aChar == 't':
                    aChar = '\t'
                elif aChar == 'r':
                    aChar = '\r'
                elif aChar == 'n':
                    aChar = '\n'
                elif aChar == 'f':
                    aChar = '\f'
        out.append(aChar)
    return ''.join(out)


def _loadConvert2_func(m):
    s = m.group()
    if s[1] == 'u':
        return s.encode().decode('unicode-escape')
    if s[1] == 't':
        return '\t'
    if s[1] == 'r':
        return '\r'
    if s[1] == 'n':
        return '\n'
    if s[1] == 'f':
        return '\f'
    return s[1]


_loadConvert2_pattern = re.compile(r'\\(u....|[^\r\n])')


def loadConvert2(string):
    '''
    regex implement
    '''
    return re.sub(_loadConvert2_pattern, _loadConvert2_func, string)


def storeConvert(unic, escapeSpace=True, escapeUnicode=True):
    '''
    Converts unicodes to encoded \\uxxxx and escapes
    special characters with a preceding slash

    Note: the return is unicode type if the input is.

    @param  unic            input string
    @param  escapeSpace     True for key, False for value (true is OK)
    @param  escapeUnicode   (boolean)
    @return                 processed string
    '''
    outBuffer = []
    for x in range(len(unic)):
        aChar = unic[x]
        # Handle common case first, selecting largest block that
        # avoids the specials below
        if ord(aChar) > 61 and ord(aChar) < 127:
            if aChar == '\\':
                outBuffer.append('\\\\')
                continue
            outBuffer.append(aChar)
            continue
        if aChar == ' ':
            if x == 0 or escapeSpace:
                outBuffer.append('\\')
            outBuffer.append(' ')
        elif aChar == '\t':
            outBuffer.append('\\t')
        elif aChar == '\n':
            outBuffer.append('\\n')
        elif aChar == '\r':
            outBuffer.append('\\r')
        elif aChar == '\f':
            outBuffer.append('\\f')
        elif aChar == '=' or aChar == ':' or aChar == '#' or aChar == '!':
            outBuffer.append('\\' + aChar)
        else:
            if (((ord(aChar) < 0x0020) or (ord(aChar) > 0x007e))
                    and escapeUnicode):
                outBuffer.append('\\u%04x' % ord(aChar))
            else:
                outBuffer.append(aChar)
    return ''.join(outBuffer)


def storeComments(writable, comments, linesep=os.linesep):
    '''
    write comments with writable.write(str).

    @param  writable    object with method write(str)
    @param  comments    string of comments
    @param  linesep     line terminator (seperator) '\\n' or '\\r' or '\\r\\n',
                        defaults os.linesep

    @see  #store(writable, properties, comments=None)
    '''
    writable.write("#")
    length = len(comments)
    current = 0
    last = 0
    while (current < length):
        c = comments[current]
        if ord(c) > 0x00ff or c == '\n' or c == '\r':
            if last != current:
                writable.write(comments[last:current])
            if ord(c) > 0x00ff:
                writable.write(c.encode('unicode-escape').decode())
            else:
                writable.write(linesep)
                if (c == '\r' and
                    current != length - 1 and
                        comments[current + 1] == '\n'):
                    current += 1
                if (current == length - 1 or
                    (comments[current + 1] != '#' and
                     comments[current + 1] != '!')):
                    writable.write("#")
            last = current + 1
        current += 1
    if last != current:
        writable.write(comments[last:current])
    writable.write(linesep)


def store(writable, properties, comments=None):
    '''
    writes the comments, properties keys and values in line-oriented format
    with writable.write(str).

    keys and values must be string.

    @param  writable    object with method write(str)
    @param  properties  a dictionary stores properties
    @param  comments    string comments to write before the properties
                        defaults None and no comment is written

    @call  #storeComments(writable, comments, linesep=os.linesep)
    @call  #storeSingle(writable, key, value, sep='=', linesep=os.linesep)

    ---detail from Java document---
     * If the comments argument is not null, then an ASCII '#'
     * character, the comments string, and a line separator are first written
     * to the output stream. Thus, the comments can serve as an
     * identifying comment. Any one of a line feed ('\\n'), a carriage
     * return ('\\r'), or a carriage return followed immediately by a line feed
     * in comments is replaced by a line separator generated by os.linesep
     * and if the next character in comments is not character '#' or
     * character '!' then an ASCII '#' is written out
     * after that line separator.
     * 
     * Then every entry in this properties is
     * written out, one per line. For each entry the key string is
     * written, then an ASCII '=', then the associated
     * element string. For the key, all space characters are
     * written with a preceding '\\' character.  For the
     * element, leading space characters, but not embedded or trailing
     * space characters, are written with a preceding '\\'
     * character. The key and element characters '#',
     * '!', '=', and ':' are written
     * with a preceding backslash to ensure that they are properly loaded.
     * 
     * After the entries have been written, 
     * *the writable object will not be closed*.

     *  *   The stream is written using the ISO 8859-1 character encoding.
     * 
     *  *   Characters not in Latin-1 in the comments are written as
     *      \\uxxxx for their appropriate unicode
     *      hexadecimal value 'xxxx'.

     *  *   Characters less than \\u0020 and characters greater
     *      than \\u007E in property keys or values are written
     *      as \\uxxxx for the appropriate hexadecimal
     *      value 'xxxx'.
    '''
    if comments != None:
        storeComments(writable, comments)
    for key, value in properties.items():
        storeSingle(writable, key, value)


def storeSingle(writable, key, value, sep='=', linesep=os.linesep):
    '''
    write key-value pair as one line with writable.write(str).

    key and value must be string.

    @param  writable    object with method write(str)
    @param  key         string of the key
    @param  value       string of the value
    @param  sep         key-value seperator, '=' or ':', defaults '='
    @param  linesep     line terminator (seperator) '\\n' or '\\r' or '\\r\\n',
                        defaults os.linesep

    @see  store(writable, properties, comments=None)
    '''
    key = storeConvert(key, True, True)
    # No need to escape embedded and trailing spaces for value, hence
    # pass False to flag.
    value = storeConvert(value, False, True)
    writable.write((key + sep + value).encode('Latin-1'))
    writable.write(linesep)


class LineReader:
    '''
    @Iterable
    read in data of one key-value pair each time from raw input.

    the return data has no leading whitespace or line terminator.

    @example
    for line in LineReader(file):
        pass

    @see  #load(iterable, properties=None)

    ---detail from Java document---
     * Read in a "logical line" from an *iterable object*, skip all comment
     * and blank lines and filter out those leading whitespace characters
     * (\\u0020, \\u0009 and \\u000c) from the beginning of a "natural line".
     '''

    def __init__(self, iterable):
        if hasNext(iterable):
            self.iterator = iterable
        elif isinstance(iterable, Iterable):
            self.iterator = iter(iterable)
        else:
            raise TypeError('\'' + str(type(iterable))
                            + '\' object is not iterable')
        self.inLimit = 0
        self.inOff = 0

    def __iter__(self):
        return self

    def next(self):
        lineBuf = []
        leng = 0

        skipWhiteSpace = True
        isCommentLine = False
        isNewLine = True
        appendedLineBegin = False
        precedingBackslash = False
        skipLF = False

        while (1):
            if self.inOff >= self.inLimit:
                try:
                    self.inByteBuf = next(self.iterator)
                    self.inLimit = len(self.inByteBuf)
                except StopIteration as e:
                    self.inLimit = -1
                self.inOff = 0
                if self.inLimit <= 0:
                    if leng == 0 or isCommentLine:
                        raise StopIteration()
                    if precedingBackslash:
                        lineBuf.pop()
                        leng -= 1
                    return ''.join(lineBuf)
            c = self.inByteBuf[self.inOff:self.inOff+1]
            self.inOff += 1
            if skipLF:
                skipLF = False
                if c == '\n':
                    continue
            if skipWhiteSpace:
                if c == ' ' or c == '\t' or c == '\f':
                    continue
                if not appendedLineBegin and (c == '\r' or c == '\n'):
                    continue
                skipWhiteSpace = False
                appendedLineBegin = False
            if isNewLine:
                isNewLine = False
                if c == '#' or c == '!':
                    isCommentLine = True
                    continue

            if c != '\n' and c != '\r':
                if leng == 0:
                    lineBuf = []
                lineBuf.append(c)
                leng += 1
                # flip the preceding backslash flag
                if c == '\\':
                    precedingBackslash = not precedingBackslash
                else:
                    precedingBackslash = False
            else:
                # reached EOL
                if isCommentLine or leng == 0:
                    isCommentLine = False
                    isNewLine = True
                    skipWhiteSpace = True
                    leng = 0
                    continue
                if self.inOff >= self.inLimit:
                    try:
                        self.inByteBuf = next(self.iterator)  # read a line
                        self.inLimit = len(self.inByteBuf)
                    except Exception as e:
                        self.inLimit = -1
                    self.inOff = 0
                    if self.inLimit <= 0:
                        if precedingBackslash:
                            lineBuf.pop()
                            leng -= 1
                        return ''.join(lineBuf)
                if precedingBackslash:
                    lineBuf.pop()
                    leng -= 1
                    # skip the leading whitespace characters in following line
                    skipWhiteSpace = True
                    appendedLineBegin = True
                    precedingBackslash = False
                    if c == '\r':
                        skipLF = True
                else:
                    return ''.join(lineBuf)


LineReader.__next__ = LineReader.next


class LineReader2:
    '''
    implement by regex

    @see  LineReader
    '''

    def __init__(self, iterable):
        self.iterator = NormalLineReader2(iterable)
        # self.inByteBuf = '' # byte[] -> string

        # in Window, package re only recognizes '\\n' as the line terminator,
        # this influence '^' and '$' so I write it by myself.

        # find a (multi-line) key-value, ignore blank lines or comments
        self.r = re.compile(
            r'(?:^|(?<=\r|\n))[ \t\f]*[^#! \t\f\r\n][\s\S]*?(?<!\\)(?:\\\\)*(?=\r\n|\r|(?<!\r)\n)')
        # find leading whiespaces each line for replacement
        self.r2 = re.compile(r'(?:^|(?<=\r|\n))[ \t\f]*')
        # find backslashes followed by the line terminator for replacement
        self.r3 = re.compile(r'\\(?:\r\n|\n|\r)')
        # find single backslashes at the end (odd number) for replacement
        self.r4 = re.compile(r'((?<!\\)(?:\\\\)*)\\$')

    def __iter__(self):
        return self

    def next(self):
        buf = ''  # buffer
        while 1:
            try:
                buf += next(self.iterator)  # read a line
            except StopIteration as e:
                if buf == '':
                    raise e
                buf += '\n'  # regex search dependent on the line terminator, it may be lost at the end of file
            m = re.search(self.r, buf)
            if m != None:
                buf = re.sub(self.r2, '', m.group())
                buf = re.sub(self.r3, '', buf)
                buf = re.sub(self.r4, lambda x: x.group(1), buf)
                if buf != '':  # may have blank line
                    return buf


LineReader2.__next__ = LineReader2.next


class NormalLineReader2:
    '''
    @Iterable
    read from raw input, read in one line each time,
    including the line terminator '\\r' or '\\n' or '\\r\\n'

    implement by regex
    '''

    def __init__(self, iterable):
        if hasNext(iterable):
            self.iterator = iterable
        elif isinstance(iterable, Iterable):
            self.iterator = iter(iterable)
        else:
            raise TypeError('\'' + str(type(iterable))
                            + '\' object is not iterable')
        self.inByteBuf = ''  # byte[] -> string

        self.r = re.compile(
            r'^(?P<line>[^\r\n]*(?:\r\n|\r(?=[^\n])|\n))(?P<left>[\s\S]*)$')

    def __iter__(self):
        return self

    def next(self):
        left = None
        while 1:
            m = re.search(self.r, self.inByteBuf)
            if m != None and m.lastindex > 1:
                self.inByteBuf = m.group('left')
                return m.group('line')
            if left:
                return left
            try:
                self.inByteBuf += next(self.iterator)
            except StopIteration as e:
                if len(self.inByteBuf) == 0:
                    raise e
                left = self.inByteBuf
                self.inByteBuf = ''


NormalLineReader2.__next__ = NormalLineReader2.next



import unittest
from time import sleep
from .event import *

num = 0
key = 'f'
key2 = 'fff'


def f(event, data):
    global num
    num = data


manager = EventManager()
manager.addListener(key, f)


class Test(unittest.TestCase):
    def setUp(self):
        manager.start()

    def tearDown(self):
        manager.stop()

    def _singleTest(self, key, n, expect, msg=''):
        manager.sendEvent(key, n)
        sleep(0.01)
        self.assertEqual(num, expect, msg=msg)

    def test_basic(self):
        self._singleTest(key, 1, 1, 'basic test')
        self._singleTest(key, 2, 2, 'basic test2')
        self._singleTest(key2, 3, 2, 'nonexist test')

    def test_stop(self):
        self._singleTest(key, 1, 1, 'basic test')

        manager.stop()
        self._singleTest(key, 2, 1, 'stop test')
        manager.start()
        sleep(0.1)
        self.assertEqual(num, 2, msg='stop test2')

    def test_clear(self):
        self._singleTest(key, 1, 1, 'basic test')

        manager.stop()
        self._singleTest(key, 2, 1, 'clear test')
        manager.clear()
        manager.start()
        sleep(0.1)
        self.assertEqual(num, 1, msg='clear test2')

    def test_remove(self):
        self._singleTest(key, 1, 1, 'basic test')

        manager.removeListener(key, f)
        self._singleTest(key, 2, 1, 'remove test')

        manager.addListener(key, f)
        self._singleTest(key, 3, 3, 'add test')


if __name__ == '__main__':
    unittest.main()



import unittest
from .path import *
import os

sep = os.path.sep


class Test(unittest.TestCase):
    def test_join(self):
        s = Path('')
        self.assertEqual(s.join('a'), 'a')
        self.assertEqual(s.join('a', '.'), 'a')
        self.assertEqual(s.join('a/b'), 'a' + sep + 'b')
        self.assertEqual(s.join('a/b', '..'), 'a')
        self.assertEqual(s.join('a\\b', '..'), 'a')
        self.assertEqual(s.join('a/b').join('..'), 'a')
        self.assertEqual(s.join('..'), '..')
        self.assertEqual(s.join('a').join('..'), '.')


if __name__ == '__main__':
    unittest.main()



import unittest
import os
import uuid
import random
from .storage import Storage

filepath = './temp_test_storage'


class Test(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        open(filepath, 'w').close()

    def test_dictFunc(self):
        storage = Storage(filepath)
        key = str(uuid.uuid4())
        value = 'test_content'
        value2 = 'test_content2'
        storage.set(key, value)
        self.assertEqual(storage[key], storage.get(key), msg='getitem')
        storage[key] = value2
        self.assertEqual(storage.get(key), value2, msg='setitem')
        del storage[key]
        self.assertEqual(key in storage, False, msg='delitem')

        storage.set(key, value2)
        storage.setdefault(key, value)
        self.assertEqual(storage.get(key), value2, msg='setdefault')
        storage.clear()
        self.assertEqual(len(storage.items()), 0, msg='empty data')
        del storage

        storage = Storage(filepath)
        self.assertEqual(len(storage.items()), 0, msg='empty data')
        del storage

        return

    def test_baseReadWrite(self):
        storage = Storage(filepath)
        self.assertEqual(len(storage.items()), 0, msg='empty data')
        key = str(uuid.uuid4())
        value = 'test_content'
        storage.set(key, value)
        self.assertEqual(len(storage.items()), 1, msg='one data')
        del storage

        storage = Storage(filepath)
        self.assertEqual(storage[key], value, msg='reopen, correct')
        self.assertEqual(len(storage.items()), 1, msg='reopen, one data')
        storage.pop(key)
        self.assertEqual(len(storage.items()), 0, msg='reopen, delete')
        del storage

        return

    def test_set(self):
        storage = Storage(filepath)
        key = str(uuid.uuid4())
        value = 'test_content'
        storage.set(key, '123')
        storage.set(key, value)
        del storage

        storage = Storage(filepath)
        self.assertEqual(storage[key], value, msg='immediate set')
        del storage

        value2 = 'test_content2'
        storage = Storage(filepath)
        storage.set(key, value2)
        del storage

        storage = Storage(filepath)
        self.assertEqual(storage[key], value2, msg='reopen set')
        del storage

        return

    def test_randomOp(self):
        testSize = 1000

        storage = Storage(filepath)
        control = {}

        i = 0
        while i < testSize:
            i += 1
            op = random.randint(1, 2)
            if op == 1:
                key = str(random.randint(0, testSize))
                value = str(random.randint(0, testSize))
                control[key] = value
                storage.set(key, value)
            elif op == 2:
                key = str(random.randint(0, testSize))
                if key in control:
                    del control[key]
                storage.pop(key)

        del storage

        storage = Storage(filepath)
        self.assertEqual(len(storage.items()), len(
            control.items()), msg='random data size')
        for k, v in storage.items():
            self.assertIn(k, control, msg='missing data ' + k)
            self.assertEqual(v, control[k], msg='random data mismatch')
        del storage

        storage = Storage(filepath)
        self.assertEqual(len(storage.items()), len(
            control.items()), msg='(2)random data size')
        for k, v in storage.items():
            self.assertIn(k, control, msg='(2)missing data ' + k)
            self.assertEqual(v, control[k], msg='(2)random data mismatch')
        del storage

        return

    def test_compress(self):
        storage = Storage(filepath)
        self.assertEqual(storage.getRecordCount(), 0, msg='empty record')
        key = str(uuid.uuid4())
        value = 'test_content'
        value2 = 'test_content2'
        storage.set(key, value)
        self.assertEqual(storage.getRecordCount(), 1, msg='one record')
        storage.set(key, value + '2')
        self.assertEqual(storage.getRecordCount(), 2, msg='two record')
        storage.set(key, value2)
        self.assertEqual(storage.getRecordCount(), 3, msg='three record')
        storage.compress()
        self.assertEqual(storage.getRecordCount(), 1,
                         msg='compress, one record')
        self.assertEqual(storage[key], value2, msg='compress, correct')
        del storage

        storage = Storage(filepath)
        storage.set(key, value)
        self.assertEqual(storage.getRecordCount(), 2,
                         msg='reopen set, two record')
        self.assertEqual(storage[key], value, msg='reopen set, correct')
        del storage

        storage = Storage(filepath)
        storage.compress()
        self.assertEqual(storage.getRecordCount(), 1,
                         msg='reopen compress, two record')
        self.assertEqual(storage[key], value, msg='reopen compress, correct')
        del storage

        return

    def tearDown(self):
        os.remove(filepath)


if __name__ == '__main__':
    unittest.main()



# coding=utf-8
'''
Custom thread objects for some kinds of loop thread, like listening thread.
In other words, the thread usually loop regularly and check for a signal
to stop its work.

Using the threading.Event object as a 'stop event' to tell the thread to stop.
'''

import threading
import inspect


def _setDaemon(thread):
    thread.setDaemon(True)


class StoppableThread(threading.Thread):
    '''
    Thread with a stop() method.

    A threading.Event will be set as an attribute,
    and passed to the target function if possible.
    with default name 'stop_event'. The thread itself has to
    check the event regularly and stop itself 
    when the event is set e.g. stop_event.is_set().

    Note: you should not change the event.
    '''

    def __init__(self, group=None, target=None, name=None, args=(),
                 kwargs=None, event_name=None, **kw):
        '''
        Arguments are same as threading.Thread with additional 'event_name',
        which specify the argument's name for the stop event.

        The event name defaults None indicate a non-strict mode, 
        and it use 'stop_event' as the name.
        Otherwise it is strict mode using the specified name.

        It check whether the target function can receive the event.

        In non-strict mode, it try using 'kwargs' first, or append to 'args'.
        It may replace the value in 'kwargs', 
        and do nothing when it is impossible to pass the event.

        In strict mode, it only use 'kwargs' and raise a ValueError 
        when there is an key confliction or it is impossible to pass the event.
        '''
        strict = event_name != None
        if not event_name:
            event_name = 'stop_event'
        self._stop_event = threading.Event()
        self.__setattr__(event_name, self._stop_event)
        if target != None:
            if kwargs == None:
                kwargs = {}
            spec = inspect.getfullargspec(target)
            # if it is possible to pass the event
            if spec.varkw != None or \
                    (len(spec.args) > len(args) and event_name in spec.args[len(args):]):
                if strict and event_name in kwargs:
                    raise ValueError('the name \'' + event_name +
                                     '\' for stop event already exists in the \'kwargs\'')
                else:
                    kwargs[event_name] = self._stop_event
            elif strict:
                raise ValueError(
                    'impossible to pass the stop event (strict mode)')
            elif spec.varargs != None or len(spec.args) > len(args):
                args = (*args, self._stop_event)
        super(StoppableThread, self).__init__(group, target, name, args,
                                              kwargs, **kw)
        return

    def stop(self, timeout=0):
        '''set the stop event to notify the thread to stop.
        and invoke join(timeout) unless the timeout is 0.
        @return whether it is alive'''
        self._stop_event.set()
        if timeout != 0:
            self.join(timeout)
        return self.isAlive()

    def stopped(self):
        '''whether the stop event is set'''
        return self._stop_event.is_set()


class ReStartableThread(object):
    '''
    Provide restartable feature in addition to StoppableThread.

    Note: start() and stop() must be called by pair, 
    otherwise there is no effect.

    Because new instance of Thread will be used after stop(),
    you should set your properties again yourself or use the processor.

    For convenience, you can set properties for the new instance directly
    after stop(). However, three methods are called on the old instance:
    join(), isAlive(), is_alive().

    The default processor will call setDaemon(True).
    '''
    _old_methods = ['join', 'isAlive', 'is_alive']

    def __init__(self, group=None, target=None, name=None, args=(),
                 kwargs=None, event_name=None, processor=_setDaemon, **kw):
        '''
        @parameter processor  a function to process the Thread after created, 
                                maybe set some properties. can be None.
                                defaults to call setDaemon(True).
        '''
        self._args = (group, target, name, args, kwargs, event_name)
        self._kw = kw
        self._processor = processor
        self._rlock = threading.RLock()
        self._thread0 = None
        self._create()

    def _create(self):
        '''create new instance of Thread and mark not started'''
        # false: the thread is a new one, not started
        # true: the thread start() was called, but may be died
        self._started = False
        self._thread = StoppableThread(*self._args, **self._kw)
        if self._processor:
            self._processor(self._thread)

    def start(self):
        '''start the thread (after stop() or just at the first time).
        @return whether it start a thread.'''
        self._rlock.acquire()
        try:
            if self._started:
                return False
            self._started = True
            self._thread.start()
            return True
        finally:
            self._rlock.release()

    def stop(self, timeout=0):
        '''set the stop event and create new instance after start().
        @return whether it is alive'''
        self._rlock.acquire()
        try:
            if not self._started:
                return False
            rtn = self._thread.stop(timeout)
            self._thread0 = self._thread
            self._create()
            return rtn
        finally:
            self._rlock.release()

    def __getattr__(self, name):
        '''get method or attributes (e.g. the event) from the thread instance if not found.'''
        self._rlock.acquire()
        try:
            if self._started or not self._thread0 or name not in self._old_methods:
                return self._thread.__getattribute__(name)
            else:
                return self._thread0.__getattribute__(name)
        finally:
            self._rlock.release()



# encoding: UTF-8
#  dP"""8b    ii    88i    88   888888
#  Ib    "    __    88 i   88   88
#   '"~_      88    88  i  88   88888
#      ""b    88    88   i 88   88
#  "booodP    88    88    i88   888888

_VERSION = (0, 1, 1)

DESCRIPTION = 'the set of common tools'
URL = 'https://github.com/SineObama/'
VERSION = '.'.join(map(str, _VERSION))
REQUIRED = []



import unittest
from .threads import *


def f1():
    pass


def f2(stop_event):
    pass


def f3(**kwargs):
    pass


run_last = 0.05
wait_exit = 1


class WhiteBox(unittest.TestCase):
    def test_parameter(self):
        def test0(Thread):
            def test(func, noerror, event_name=None, kwargs=None):
                try:
                    Thread(target=func, event_name=event_name, kwargs=kwargs)
                except ValueError as e:
                    noerror = not noerror
                self.assertTrue(noerror)
            test(f1, True)
            test(f1, False, 'stop_event')
            test(f1, True, kwargs={'stop_event': None})
            test(f2, True)
            test(f2, True, 'stop_event')
            test(f2, False, 'stop_event', {'stop_event': None})
            test(f3, True)
            test(f3, True, 'stop_event')
            test(f3, False, 'stop_event', {'stop_event': None})
        test0(StoppableThread)
        test0(ReStartableThread)

    def test_stop(self):
        def test(Thread):
            stopped = [False]

            def f(stop_event):
                while 1:
                    if stop_event.is_set():
                        break
                stopped[0] = True
            t = Thread(target=f)
            t.start()
            t.join(run_last)
            t.stop()
            t.join(wait_exit)
            self.assertTrue(stopped[0])
        test(StoppableThread)
        test(ReStartableThread)

    def test_restart(self):
        s = [0]

        def f(stop_event):
            s[0] += 1
            while 1:
                if stop_event.is_set():
                    break
            s[0] += 10

        t = ReStartableThread(target=f)
        t.start()
        t.join(run_last)
        self.assertEqual(s[0], 1)
        t.stop()
        t.join(wait_exit)
        self.assertEqual(s[0], 11)
        t.start()
        t.join(run_last)
        self.assertEqual(s[0], 12)
        t.stop()
        t.join(wait_exit)
        self.assertEqual(s[0], 22)

    def test_other(self):
        t = ReStartableThread(target=f2)
        t.stop()
        t.stop()
        t.start()
        t.start()
        t.stop()


if __name__ == '__main__':
    unittest.main()



# encoding: UTF-8
import csv
import os
import sys
import threading
import logging
from .path import Path

_logger = logging.getLogger(__name__)


class FilterNullReader:
    '''过滤null字符防止 csv 读取报错'''

    def __init__(self, iterable):
        if hasattr(iterable, '__next__'):
            self.iterator = iterable
        else:
            self.iterator = iter(iterable)

    def __iter__(self):
        return self

    def __next__(self):
        val = next(self.iterator)
        if isinstance(val, str):
            val = val.replace('\x00', '')
        return val


class Storage(object):
    '''
    提供简单可靠的字符串数据持久化功能。

    数据类似字符串字典，可更新、删除。
    可以像 dict 一样使用，其中涉及数据修改的函数支持 clear pop setdefault 。
    支持多线程，但不要使用同一文件创建多个实例。

    使用 csv 格式读写文件，可设置编码 encoding 。
    平时只追加数据到文件末尾，希望能避免系统异常时覆写数据。
    可以适时手动压缩数据，将以比较可靠谨慎的方式覆写文件。

    为了减少异常文件的影响，解析 csv 前过滤掉了null字符。
    '''

    def __init__(self, filePath, encoding=sys.stdout.encoding, load=True, logger=None):
        '''
        创建仓库实例，绑定一个 csv 数据文件。
        @param load 立即加载数据
        '''
        self.encoding = encoding
        self._filePath = Path(os.path.abspath(filePath))
        self._filename = os.path.basename(filePath)
        self._dirPath = self._filePath.join('..')
        self._lock = threading.RLock()
        self._data = {}
        self._recordCount = 0
        self.logger = logger if logger else _logger
        if load:
            self.reload()
        return

    def reload(self):
        '''重新从文件加载数据'''
        self._lock.acquire()
        try:
            if not os.path.exists(self._dirPath):
                os.makedirs(self._dirPath)
            if os.path.isfile(self._filePath):
                self._readData(self._filePath)
        finally:
            self._lock.release()

    def _readData(self, path):
        '''读取异常情况，目前只能中止，保留已读结果。'''
        data = {}
        recordCount = 0
        try:
            with open(path, 'r', encoding=self.encoding) as file:
                for line in csv.reader(FilterNullReader(file)):
                    if len(line) < 1:
                        continue
                    try:
                        version = line[0]
                        if version == '1':
                            op = line[1]
                            key = line[2]
                            if op == 'u':
                                data[key] = line[3]
                            elif op == 'd':
                                del data[key]
                            else:
                                raise Exception(
                                    'unsupported operation "%s"' % (op))
                        else:
                            raise Exception(
                                'unsupported data version "%s"' % (version))
                        recordCount += 1
                    except Exception:
                        pass
        except Exception as e:
            self.logger.warning(e)
            raise
        finally:
            # 保存已读数据
            self._data = data
            self._recordCount = recordCount

    def compress(self):
        '''压缩数据文件，相当于删除历史记录，每条数据只保留他本身。
        确保可靠：
        1.把数据写出到临时文件
        2.重新读取原文件数据，与内存数据对比，无误则继续
        3.用多次重命名文件的方式慎重地覆盖原文件'''
        self._lock.acquire()
        try:
            # 1.写出到临时文件
            compName = self._filename + '.compress'
            compPath = self._dirPath.join(compName)
            with open(compPath, 'w', newline='', encoding=self.encoding) as compFile:
                compWriter = csv.writer(compFile)
                for key, item in self._data.items():
                    compWriter.writerow(['1', 'u', key, item])

            # 2.重新读原文件进行对比
            data = self._data
            try:
                self._readData(self._filePath)
            except Exception as e:
                self.logger.info(
                    'Exception reading origin file while compressing: ' + e)
            items = data.items()
            originItems = self._data.items()
            if len(items) != len(originItems):
                raise RuntimeError('compress fail: data length not equals')
            for key, item in originItems:
                if key not in data:
                    raise RuntimeError(
                        'compress fail: miss data key "%s"' % key)
                if item != data[key]:
                    raise RuntimeError('compress fail: data mismatch for key "%s" (%s, %s)' % (
                        key, item, data[key]))

            # 3.多次重命名文件覆盖
            # 先把原文件重命名为.bak的备份文件（考虑文件已存在）
            postFix = 0
            bakName = self._filename + '.bak'
            bakPath = self._dirPath.join(bakName)
            while os.path.exists(bakPath):
                bakPath = self._dirPath.join(bakName + '.' + str(postFix))
                postFix += 1
            os.rename(self._filePath, bakPath)
            # 再把新的压缩文件重命名为原文件名，最后再删除备份文件
            os.rename(compPath, self._filePath)
            os.remove(bakPath)

            self.reload()
        finally:
            self._lock.release()

    def clear(self):
        self._lock.acquire()
        try:
            for key in list(self._data.keys()):
                self.pop(key)
        finally:
            self._lock.release()

    def set(self, key, item):
        '''更新一条数据'''
        self._lock.acquire()
        try:
            with open(self._filePath, 'a', newline='', encoding=self.encoding) as file:
                writer = csv.writer(file)
                writer.writerow(['1', 'u', key, item])
                self._recordCount += 1
                self._data[key] = item
        finally:
            self._lock.release()

    def setdefault(self, key, item=''):
        self._lock.acquire()
        try:
            if key in self._data:
                return self._data[key]
            self.set(key, item)
            return item
        finally:
            self._lock.release()

    def pop(self, key):
        '''删除一条数据'''
        self._lock.acquire()
        try:
            if key not in self._data:
                return
            with open(self._filePath, 'a', newline='', encoding=self.encoding) as file:
                writer = csv.writer(file)
                writer.writerow(['1', 'd', key])
                self._recordCount += 1
                del self._data[key]
        finally:
            self._lock.release()

    def getRecordCount(self):
        '''获取数据文件的记录条数'''
        return self._recordCount

    def getUtilization(self):
        '''获取数据文件使用率，即最终的数据量占记录数的百分比'''
        return float(len(self._data.items())) / self._recordCount

    def __getitem__(self, key):
        return self._data.get(key)

    def __setitem__(self, key, value):
        return self.set(key, value)

    def __delitem__(self, key):
        return self.pop(key)

    def __getattr__(self, key):
        return getattr(self._data, key)

    def __contains__(self, key):
        return key in self._data


__all__ = ['Storage']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pip install twine

# The only thing you have to do is to
# rewrite meta-data in __meta__.py as these default value below
# and write this module_name
MODULE_NAME = 'utils'
# Personal namespace package
NAMESPACE = 'sine'
DESCRIPTION = 'package description'
URL = 'https://github.com/SineObama/'
EMAIL = '714186139@qq.com'
AUTHOR = 'Xian Zheng'
REQUIRES_PYTHON = '>=3.6.0'
VERSION = '0.0.1'
REQUIRED = [
    # What packages are required for this module to be executed?
    # 'requests', 'maya', 'records',
]
# If you do change the License, remember to change the Trove Classifier for that!
LICENSE='MIT'
CLASSIFIERS=[
    # Trove classifiers
    # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: Implementation :: CPython',
    'Programming Language :: Python :: Implementation :: PyPy'
]

import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Replace globals with value in __meta__.py
# Load the package's __meta__.py module as a dictionary.
NAME = NAMESPACE + '.' + MODULE_NAME
here = os.path.abspath(os.path.dirname(__file__))
meta = {}
with open(os.path.join(here, NAMESPACE, MODULE_NAME, '__meta__.py')) as f:
    exec(f.read(), meta)
for key, value in meta.items():
    exec(key + ' = meta[\'' + key + '\']')

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    LONG_DESCRIPTION = '\n' + f.read()

class MyCommand(Command):
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('>>>>{0}'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

class UploadCommand(MyCommand):
    """Support setup.py upload."""

    description = 'Build and publish the package.'

    def run(self):
        try:
            self.status('Removing previous builds...')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source distribution...')
        os.system('{0} setup.py sdist'.format(sys.executable))

        # self.status('Building executable installer for MS Windows...')
        # os.system('{0} setup.py bdist_wininst'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine...')
        os.system('twine upload dist/*')

        self.status('Pushing git tags...')
        os.system('git tag v{0}'.format(VERSION))
        os.system('git push --tags')
        
        sys.exit()

class ReinstallCommand(MyCommand):
    """repack the package and reinstall locally."""
    def run(self):
        try:
            self.status('Removing previous builds...')
            rmtree(os.path.join(here, 'build'))
        except OSError:
            pass

        self.status('Building...')
        os.system('{0} setup.py build'.format(sys.executable))

        self.status('Installing...')
        os.system('{0} setup.py install'.format(sys.executable))
        
        sys.exit()

class UninstallCommand(MyCommand):
    def run(self):
        self.status('Uninstalling the package locally')
        os.system('pip uninstall ' + NAME)
        
        sys.exit()

# Where the magic happens:
setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=('tests',)),
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],

    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },
    namespace_packages=[NAMESPACE],
    install_requires=REQUIRED,
    include_package_data=True,
    license=LICENSE,
    classifiers=CLASSIFIERS,
    # $ setup.py publish support.
    cmdclass={
        'upload': UploadCommand,
        'rei': ReinstallCommand,
        'uni': UninstallCommand,
    },
)



# coding=utf-8

import unittest
from .properties import *


class WhiteBox(unittest.TestCase):
    def test_NormalLineReader2(self):
        s = '\
a\n\
b\\\n\
c\\\\\r\
\r\n\
z\
'
        result = [
            'a\n',
            'b\\\n',
            'c\\\\\r',
            '\r\n',
            'z'
        ]
        i = 0
        for line in NormalLineReader2(s):
            self.assertEqual(result[i], line)
            i += 1
        i = 0
        for line in NormalLineReader2([s]):
            self.assertEqual(result[i], line)
            i += 1

    def test_LineReader_LineReader2(self):
        s = '\
a = b\r\n\
   \t\f\\\r\n\r\n\n\
   \t\fb = b\n\
! c\n\
# c\n\
# c\\\n\
 c = \\\r\n  b\\\\\r\n\
   \td =  \tb\n\
e = b\\\r\nc\n\
  z = b\
'
        result = [
            'a = b',  # normal with flag skipLF
            # skipWhiteSpace flow, escape line terminator, blank line
            'b = b',  # leading whitespaces
            # continous comment lines
            # comment can not escape line terminator
            'c = b\\\\',  # escape line terminator, even number of '\'
            'd =  \tb',
            # --- regex compare tests ---
            'e = bc',  # \r\n together
            'z = b',  # ends with no newlines
        ]
        i = 0
        for line in LineReader(s):
            self.assertEqual(result[i], line)
            i += 1
        i = 0
        for line in LineReader([s]):
            self.assertEqual(result[i], line)
            i += 1
        i = 0
        for line in LineReader2(s):
            self.assertEqual(result[i], line)
            i += 1
        i = 0
        for line in LineReader2([s]):
            self.assertEqual(result[i], line)
            i += 1

    def test_LineReader_LineReader2_2(self):
        s = '\
a = b\n\
# c\\\n\
   \te = b\\\\\\\
'
        result = [
            'a = b',  # ends with comment line and '\'
            'e = b\\\\',  # ends with '\'
        ]
        i = 0
        for line in LineReader(s):
            self.assertEqual(result[i], line)
            i += 1
        i = 0
        for line in LineReader([s]):
            self.assertEqual(result[i], line)
            i += 1
        i = 0
        for line in LineReader2(s):
            self.assertEqual(result[i], line)
            i += 1
        i = 0
        for line in LineReader2([s]):
            self.assertEqual(result[i], line)
            i += 1

    def test_loadSingle_loadSingle2(self):
        self.assertEqual(loadSingle('key value'), ('key', 'value'))
        self.assertEqual(loadSingle(' \t\fkey value'), ('key', 'value'))

        self.assertEqual(loadSingle2('key value'), ('key', 'value'))
        self.assertEqual(loadSingle2(' \t\fkey value'), ('key', 'value'))

    def test_identifyKeyValue(self):
        def test(string, v1, v2):
            result = identifyKeyValue(string)
            self.assertEqual((v1, v2), result, string + ' -> ' +
                             str(result) + ' but except ' + str((v1, v2)))

        # single seperator
        test('key=value', 3, 4)
        test('key:value', 3, 4)
        test('key value', 3, 4)
        test('key\tvalue', 3, 4)
        test('key\fvalue', 3, 4)

        # escaped
        test('key \\=value', 3, 4)
        test('key \\ value', 3, 4)

        # whitespaces around '='
        test('key =value', 3, 5)
        test('key= value', 3, 5)
        test('key  = value', 3, 7)

        # continous whitespaces as seperator
        test('key    value', 3, 7)
        test('key  \t value', 3, 7)

        # special cases
        test('key:=value', 3, 4)
        test('key:= value', 3, 4)

    def test_loadConvert_loadConvert2(self):
        self.assertEqual(loadConvert('s'), 's')
        self.assertEqual(loadConvert('\\u005c'), '\\')
        self.assertEqual(loadConvert('\\u4e2d'), u'中')
        self.assertEqual(loadConvert('\\t'), '\t')
        self.assertEqual(loadConvert('\\r'), '\r')
        self.assertEqual(loadConvert('\\n'), '\n')
        self.assertEqual(loadConvert('\\f'), '\f')
        self.assertEqual(loadConvert('\\b'), 'b')
        self.assertEqual(loadConvert('\\a'), 'a')

        self.assertEqual(loadConvert2('s'), 's')
        self.assertEqual(loadConvert2('\\u005c'), '\\')
        self.assertEqual(loadConvert2('\\u4e2d'), u'中')
        self.assertEqual(loadConvert2('\\t'), '\t')
        self.assertEqual(loadConvert2('\\r'), '\r')
        self.assertEqual(loadConvert2('\\n'), '\n')
        self.assertEqual(loadConvert2('\\f'), '\f')
        self.assertEqual(loadConvert2('\\b'), 'b')
        self.assertEqual(loadConvert2('\\a'), 'a')

    def test_storeConvert(self):
        self.assertEqual(storeConvert('\\'), '\\\\')
        self.assertEqual(storeConvert('  '), '\\ \\ ')
        self.assertEqual(storeConvert(' ', False), '\\ ')
        self.assertEqual(storeConvert('  ', False), '\\  ')
        self.assertEqual(storeConvert('\t'), '\\t')
        self.assertEqual(storeConvert('\n'), '\\n')
        self.assertEqual(storeConvert('\r'), '\\r')
        self.assertEqual(storeConvert('\f'), '\\f')
        self.assertEqual(storeConvert('='), '\\=')
        self.assertEqual(storeConvert(':'), '\\:')
        self.assertEqual(storeConvert('#'), '\\#')
        self.assertEqual(storeConvert('!'), '\\!')
        self.assertEqual(storeConvert(u'\u0019'), '\\u0019')
        self.assertEqual(storeConvert(u'\u00ff'), '\\u00ff')
        self.assertEqual(storeConvert(u'\u00ff', escapeUnicode=False), u'\xff')
        self.assertEqual(storeConvert(
            u'\u2eff', escapeUnicode=False), u'\u2eff')
        self.assertEqual(storeConvert(u'中'), '\\u4e2d')

    def test_storeComments(self):
        import os
        ls = os.linesep

        class Writer():
            def __init__(self):
                self.buf = ''

            def write(self, s):
                self.buf += s

        def test(string, *args):
            w = Writer()
            storeComments(w, string)
            lines = list(args)
            lines.append('')
            self.assertEqual(w.buf, ls.join(lines))
        test('a', '#a')
        test(u'\u4e2d', '#\\u4e2d')
        test('\r', '#', '#')
        test('\n', '#', '#')
        test('\r\n', '#', '#')
        test('\r\n#', '#', '#')
        test('\r\n!', '#', '!')
        test('\r\n #', '#', '# #')


if __name__ == '__main__':
    unittest.main()



# encoding: UTF-8
import logging
from threading import Thread
from queue import Queue, Empty
from .threads import ReStartableThread


class EventManager:
    '''
    以 key 为键标识每个事件，对一个 key 可以添加或移除多个监听器。
    发布事件时传入 key 和数据 data ，后者将会传给监听器。

    可指定 logger 记录日志，否则默认使用 __name__ 的 logger 。
    '''

    def __init__(self, logger=None, eventBlockTime=0.1):
        self._eventQueue = Queue()
        self._listenThread = ReStartableThread(target=self._run)
        # 保存对应的事件的响应函数，每个键对应一个集合
        self._map = {}
        self.eventBlockTime = eventBlockTime
        self.logger = logger if logger else logging.getLogger(__name__)
        self.setDaemon(True)

    def _run(self, stop_event):
        while not stop_event.is_set():
            try:
                event = self._eventQueue.get(block=True, timeout=self.eventBlockTime)
                if stop_event.is_set():
                    return
                self.logger.info('process event. key=%s' %
                                  (str(event[0])))
                self._process(event)
            except Empty:
                pass

    def _process(self, event):
        if event[0] in self._map:
            for listener in self._map[event[0]]:
                def sub(listener=listener):
                    try:
                        listener(event[0], *event[1], **event[2])
                    except Exception as e:
                        self.logger.warn('listener exception. listener=%s, exception=%s, event_key=%s' %
                                         (str(listener), str(e), str(event[0])))
                        raise
                thread = Thread(target=sub)
                thread.setDaemon(self._daemon)
                thread.start()

    def setDaemon(self, daemon):
        '''修改处理函数是否为守护线程'''
        self._daemon = True if daemon else False
        self.logger.info('daemon set to ' + str(self._daemon))

    def start(self):
        '''开始事件监听。'''
        self._listenThread.start()
        self.logger.info('listen start')

    def stop(self):
        '''停止事件监听。'''
        self._listenThread.stop()
        self._listenThread.join(1)
        self.logger.info('listen stop')

    def clear(self):
        '''清空事件。'''
        self._eventQueue.queue.clear()

    def addListener(self, key, listener):
        self.logger.info('add listener. key=%s, listener=%s' %
                          (str(key), str(listener)))
        self._map.setdefault(key, set()).add(listener)

    def removeListener(self, key, listener):
        try:
            self._map[key].remove(listener)
            self.logger.info('removed listener. key=%s, listener=%s' % (
                str(key), str(listener)))
        except KeyError:
            pass

    def sendEvent(self, key, *args, **kwargs):
        '''发布事件。以 key 标识；data 为数据，将会传给监听器。'''
        self.logger.info('send event. key=%s' %
                          (str(key)))
        self._eventQueue.put((key, args, kwargs))


__all__ = ['EventManager']
